/*
 * Copyright (c) 2014, Sonos, Inc.
 *
 * SPDX-License-Identifier: GPL-2.0
 *
 * sonos_signature_sign.h: Sonos digital signature format
 *                         creation and serialization
 */

#ifndef SONOS_SIGNATURE_SIGN_H
#define SONOS_SIGNATURE_SIGN_H

#include "sonos_signature.h"
#include "sonos_signature_common.h"
#include "sonos_attr.h"

#ifdef __cplusplus
extern "C" {
#endif


size_t sonosSignatureSerialize(const SonosSignature *sig,
                               uint8_t *buf, size_t bufLen);

typedef int (*SonosRawSignCallback)(SonosSigningKey_t key,
                                    SonosSignatureAlg_t signAlg,
                                    SonosDigestAlg_t digestAlg,
                                    const uint8_t *digest, size_t digestLen,
                                    uint8_t *signature, uint32_t *pSigLen);

int sonosSignatureCreate(SonosSignature *sig,
                         SonosHashCallback hash,
                         SonosRawSignCallback rawSign,
                         SonosDigestAlg_t digestAlg,
                         SonosSignatureAlg_t sigAlg,
                         SonosSigningKey_t key,
                         SonosKeyIdentifierScheme_t keyIdScheme,
                         const uint8_t *keyId,
                         uint32_t keyIdLen,
                         const uint8_t *msg, size_t msgLen,
                         SonosContentType_t ct,
                         SonosSrkRevoke_t srkRevoke);

int sonosSignatureCreateFromDigest(SonosSignature *sig,
                                   SonosHashCallback hash,
                                   SonosRawSignCallback rawSign,
                                   SonosDigestAlg_t digestAlg,
                                   SonosSignatureAlg_t sigAlg,
                                   SonosSigningKey_t key,
                                   SonosKeyIdentifierScheme_t keyIdScheme,
                                   const uint8_t *keyId,
                                   uint32_t keyIdLen,
                                   const uint8_t *digest, size_t digestLen,
                                   SonosContentType_t ct,
                                   SonosSrkRevoke_t srkRevoke);

#ifdef __cplusplus
}
#endif

#endif
